#ifndef ATPSECTOR_H
#define ATPSECTOR_H

/*
   AtpSector.h - data of an sector according to the ATP format

   Copyright (C) 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <iostream>

#include "RCPtr.h"
#include "RefCounted.h"
#include "ChunkReader.h"
#include "ChunkWriter.h"

class AtpSector: public RefCounted {
public:
	AtpSector(
		size_t id,
		size_t data_len,
		const unsigned char* data,
		size_t pos,
		size_t time_len,
		unsigned char status=255);

	virtual ~AtpSector();

	// get sector ID
	inline size_t GetID() const;

	// get absolute sector position on track (in usec)
	inline size_t GetPosition() const;

	// get time length of sector on track (in usec)
	inline size_t GetTimeLength() const;

	// get error status of sector
	inline unsigned char GetSectorStatus() const;

	// set error status of sector
	inline void SetSectorStatus(unsigned char status);

	// get lenght of data block
	inline size_t GetDataLength() const;

	// get data block of sector
	bool GetData(unsigned char* dest, size_t len) const;

	inline const unsigned char* GetDataPtr() const;

	// set data block of sector
	bool SetData(const unsigned char* src, size_t len);
	
	void Dump(std::ostream& os, size_t indentlevel=0) const;

	// create ATP "SECT" chunk from internal data
	RCPtr<ChunkWriter> BuildSectorChunk() const;

	// set internal data from ATP "SECT" chunk
	bool InitFromSectorChunk(RCPtr<ChunkReader> chunk, bool beQuiet);

private:
	friend class AtpTrack;
	AtpSector();

private:
	size_t fID;
	size_t fDataLength;
	unsigned char* fSectorData;
	size_t fPosition;
	size_t fTimeLength;
	unsigned char fSectorStatus;
};

inline size_t AtpSector::GetPosition() const
{
	return fPosition;
}

inline size_t AtpSector::GetID() const
{
	return fID;
}

inline unsigned char AtpSector::GetSectorStatus() const
{
	return fSectorStatus;
}

inline void AtpSector::SetSectorStatus(unsigned char status)
{
	fSectorStatus = status;
}

inline size_t AtpSector::GetDataLength() const
{
	return fDataLength;
}

inline size_t AtpSector::GetTimeLength() const
{
	return fTimeLength;
}

inline const unsigned char* AtpSector::GetDataPtr() const
{
	return fSectorData;
}

#endif
