/*
   adir.cpp - list directory of ATR image

   Copyright (C) 2002, 2003 Matthias Reichl <hias@horus.com>

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
*/

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>

#include "AtrMemoryImage.h"
#include "SIOTracer.h"
#include "FileTracer.h"

static void print_directory(const RCPtr<const AtrImage>& image, unsigned int columns)
{
	RCPtr<Dos2Dir> dir = image->GetDos2Directory();
	if (dir.IsNull()) {
		printf("error reading directory\n");
		return;
	}

	if (columns < 1) {
		columns = 1;
	}
	unsigned int i;
	for (i=0;i<dir->GetNumberOfFiles();i++) {
		printf("%s", dir->GetFile(i));
		if ( i % columns == (columns - 1)) {
			printf("\n");
		}
	}
	if (i % columns != 0) {
		printf("\n");
	}
	printf("%d free sectors\n", dir->GetFreeSectors());
}

int main(int argc, char**argv)
{
	int columns=4;
	char* filename;
	RCPtr<AtrMemoryImage> image;
	int idx=1;

	SIOTracer* sioTracer = SIOTracer::GetInstance();
	{
		RCPtr<FileTracer> tracer(new FileTracer(stderr));
		sioTracer->AddTracer(tracer);
		sioTracer->SetTraceGroup(SIOTracer::eTraceInfo, true, tracer);
		sioTracer->SetTraceGroup(SIOTracer::eTraceDebug, true, tracer);
	}


	if (argc < 2) {
		goto usage;
	}

	if (argv[idx][0]=='-') {
		columns = atoi(argv[1]+1);
		if (columns < 1 || columns > 64) {
			printf("illegal number of columns\n");
			goto usage;
		}
		idx++;
	}
	if (argc <= idx) {
		goto usage;
	}

	image = new AtrMemoryImage;
	while (idx < argc) {
		filename = argv[idx];
		if (!image->ReadImageFromFile(filename)) {
			printf("reading image '%s' failed\n",filename);
		} else {
			printf("\ndirectory of image '%s'\n\n",filename);
			print_directory(image, columns);
		}
		idx++;
	}

	SIOTracer::GetInstance()->RemoveAllTracers();
	return 0;
usage:
	printf("adir V0.01 (c) 2003 by Matthias Reichl\n");
	printf("usage: adir [-<columns>] filename...\n");
	SIOTracer::GetInstance()->RemoveAllTracers();
	return 1;
}
